"""
A bunch of cool widgets from protovis (under the 'custom' heading in their
examples page).

Get this source from http://github.com/ralphbean/tw2.protovis.custom

Protovis itself can be found here:  http://vis.stanford.edu/protovis/
"""

from widgets import *
